# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/model/v1/health.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/model/v1/health.proto',
  package='v1.model',
  syntax='proto3',
  serialized_options=b'Z)github.com/FormantIO/genproto/go/v1/model',
  serialized_pb=b'\n\x1cprotos/model/v1/health.proto\x12\x08v1.model\"8\n\x06Health\x12.\n\x06status\x18\x01 \x01(\x0e\x32\x16.v1.model.HealthStatusR\x06status\"u\n\x07\x42\x61ttery\x12\x1e\n\npercentage\x18\x01 \x01(\x01R\npercentage\x12\x18\n\x07voltage\x18\x02 \x01(\x01R\x07voltage\x12\x18\n\x07\x63urrent\x18\x03 \x01(\x01R\x07\x63urrent\x12\x16\n\x06\x63harge\x18\x04 \x01(\x01R\x06\x63harge*`\n\x0cHealthStatus\x12\x12\n\x0eHEALTH_UNKNOWN\x10\x00\x12\x16\n\x12HEALTH_OPERATIONAL\x10\x01\x12\x12\n\x0eHEALTH_OFFLINE\x10\x02\x12\x10\n\x0cHEALTH_ERROR\x10\x03\x42+Z)github.com/FormantIO/genproto/go/v1/modelb\x06proto3'
)

_HEALTHSTATUS = _descriptor.EnumDescriptor(
  name='HealthStatus',
  full_name='v1.model.HealthStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HEALTH_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEALTH_OPERATIONAL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEALTH_OFFLINE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEALTH_ERROR', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=219,
  serialized_end=315,
)
_sym_db.RegisterEnumDescriptor(_HEALTHSTATUS)

HealthStatus = enum_type_wrapper.EnumTypeWrapper(_HEALTHSTATUS)
HEALTH_UNKNOWN = 0
HEALTH_OPERATIONAL = 1
HEALTH_OFFLINE = 2
HEALTH_ERROR = 3



_HEALTH = _descriptor.Descriptor(
  name='Health',
  full_name='v1.model.Health',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='v1.model.Health.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=42,
  serialized_end=98,
)


_BATTERY = _descriptor.Descriptor(
  name='Battery',
  full_name='v1.model.Battery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='percentage', full_name='v1.model.Battery.percentage', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='percentage', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='voltage', full_name='v1.model.Battery.voltage', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='voltage', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='current', full_name='v1.model.Battery.current', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='current', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='charge', full_name='v1.model.Battery.charge', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='charge', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=217,
)

_HEALTH.fields_by_name['status'].enum_type = _HEALTHSTATUS
DESCRIPTOR.message_types_by_name['Health'] = _HEALTH
DESCRIPTOR.message_types_by_name['Battery'] = _BATTERY
DESCRIPTOR.enum_types_by_name['HealthStatus'] = _HEALTHSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Health = _reflection.GeneratedProtocolMessageType('Health', (_message.Message,), {
  'DESCRIPTOR' : _HEALTH,
  '__module__' : 'protos.model.v1.health_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Health)
  })
_sym_db.RegisterMessage(Health)

Battery = _reflection.GeneratedProtocolMessageType('Battery', (_message.Message,), {
  'DESCRIPTOR' : _BATTERY,
  '__module__' : 'protos.model.v1.health_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Battery)
  })
_sym_db.RegisterMessage(Battery)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from formant.protos.agent.v1 import agent_pb2 as protos_dot_agent_dot_v1_dot_agent__pb2
from formant.protos.model.v1 import datapoint_pb2 as protos_dot_model_dot_v1_dot_datapoint__pb2
from formant.protos.model.v1 import intervention_pb2 as protos_dot_model_dot_v1_dot_intervention__pb2
from formant.protos.model.v1 import math_pb2 as protos_dot_model_dot_v1_dot_math__pb2


class AgentStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.StreamData = channel.stream_unary(
        '/v1.agent.Agent/StreamData',
        request_serializer=protos_dot_model_dot_v1_dot_datapoint__pb2.Datapoint.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.StreamDataResponse.FromString,
        )
    self.PostData = channel.unary_unary(
        '/v1.agent.Agent/PostData',
        request_serializer=protos_dot_model_dot_v1_dot_datapoint__pb2.Datapoint.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataResponse.FromString,
        )
    self.PostDataMulti = channel.unary_unary(
        '/v1.agent.Agent/PostDataMulti',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataMultiRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataMultiResponse.FromString,
        )
    self.CreateEvent = channel.unary_unary(
        '/v1.agent.Agent/CreateEvent',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.CreateEventRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.CreateEventResponse.FromString,
        )
    self.CreateInterventionRequest = channel.unary_unary(
        '/v1.agent.Agent/CreateInterventionRequest',
        request_serializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.SerializeToString,
        response_deserializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.FromString,
        )
    self.GetInterventionRequest = channel.unary_unary(
        '/v1.agent.Agent/GetInterventionRequest',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetInterventionRequestRequest.SerializeToString,
        response_deserializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.FromString,
        )
    self.GetInterventionResponse = channel.unary_unary(
        '/v1.agent.Agent/GetInterventionResponse',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetInterventionResponseRequest.SerializeToString,
        response_deserializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionResponse.FromString,
        )
    self.GetStreamsConfiguration = channel.unary_unary(
        '/v1.agent.Agent/GetStreamsConfiguration',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetStreamsConfigurationRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetStreamsConfigurationResponse.FromString,
        )
    self.GetApplicationConfiguration = channel.unary_unary(
        '/v1.agent.Agent/GetApplicationConfiguration',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetApplicationConfigurationRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetApplicationConfigurationResponse.FromString,
        )
    self.GetAgentConfiguration = channel.unary_unary(
        '/v1.agent.Agent/GetAgentConfiguration',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetAgentConfigurationRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetAgentConfigurationResponse.FromString,
        )
    self.Health = channel.unary_unary(
        '/v1.agent.Agent/Health',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.HealthRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.HealthResponse.FromString,
        )
    self.GetCommandRequest = channel.unary_unary(
        '/v1.agent.Agent/GetCommandRequest',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestResponse.FromString,
        )
    self.GetCommandRequestStream = channel.unary_stream(
        '/v1.agent.Agent/GetCommandRequestStream',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestStreamRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestStreamResponse.FromString,
        )
    self.SendCommandResponse = channel.unary_unary(
        '/v1.agent.Agent/SendCommandResponse',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.SendCommandResponseRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.SendCommandResponseResponse.FromString,
        )
    self.PostTransformFrame = channel.unary_unary(
        '/v1.agent.Agent/PostTransformFrame',
        request_serializer=protos_dot_model_dot_v1_dot_math__pb2.TransformFrame.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostTransformFrameResponse.FromString,
        )
    self.SetBaseFrameID = channel.unary_unary(
        '/v1.agent.Agent/SetBaseFrameID',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.SetBaseFrameIDRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.SetBaseFrameIDResponse.FromString,
        )
    self.ClearTransformTree = channel.unary_unary(
        '/v1.agent.Agent/ClearTransformTree',
        request_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.ClearTransformTreeRequest.SerializeToString,
        response_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.ClearTransformTreeResponse.FromString,
        )


class AgentServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def StreamData(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PostData(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PostDataMulti(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateEvent(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateInterventionRequest(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetInterventionRequest(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetInterventionResponse(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetStreamsConfiguration(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetApplicationConfiguration(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetAgentConfiguration(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Health(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetCommandRequest(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetCommandRequestStream(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SendCommandResponse(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PostTransformFrame(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SetBaseFrameID(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ClearTransformTree(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AgentServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'StreamData': grpc.stream_unary_rpc_method_handler(
          servicer.StreamData,
          request_deserializer=protos_dot_model_dot_v1_dot_datapoint__pb2.Datapoint.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.StreamDataResponse.SerializeToString,
      ),
      'PostData': grpc.unary_unary_rpc_method_handler(
          servicer.PostData,
          request_deserializer=protos_dot_model_dot_v1_dot_datapoint__pb2.Datapoint.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataResponse.SerializeToString,
      ),
      'PostDataMulti': grpc.unary_unary_rpc_method_handler(
          servicer.PostDataMulti,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataMultiRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostDataMultiResponse.SerializeToString,
      ),
      'CreateEvent': grpc.unary_unary_rpc_method_handler(
          servicer.CreateEvent,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.CreateEventRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.CreateEventResponse.SerializeToString,
      ),
      'CreateInterventionRequest': grpc.unary_unary_rpc_method_handler(
          servicer.CreateInterventionRequest,
          request_deserializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.FromString,
          response_serializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.SerializeToString,
      ),
      'GetInterventionRequest': grpc.unary_unary_rpc_method_handler(
          servicer.GetInterventionRequest,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetInterventionRequestRequest.FromString,
          response_serializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionRequest.SerializeToString,
      ),
      'GetInterventionResponse': grpc.unary_unary_rpc_method_handler(
          servicer.GetInterventionResponse,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetInterventionResponseRequest.FromString,
          response_serializer=protos_dot_model_dot_v1_dot_intervention__pb2.InterventionResponse.SerializeToString,
      ),
      'GetStreamsConfiguration': grpc.unary_unary_rpc_method_handler(
          servicer.GetStreamsConfiguration,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetStreamsConfigurationRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetStreamsConfigurationResponse.SerializeToString,
      ),
      'GetApplicationConfiguration': grpc.unary_unary_rpc_method_handler(
          servicer.GetApplicationConfiguration,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetApplicationConfigurationRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetApplicationConfigurationResponse.SerializeToString,
      ),
      'GetAgentConfiguration': grpc.unary_unary_rpc_method_handler(
          servicer.GetAgentConfiguration,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetAgentConfigurationRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetAgentConfigurationResponse.SerializeToString,
      ),
      'Health': grpc.unary_unary_rpc_method_handler(
          servicer.Health,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.HealthRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.HealthResponse.SerializeToString,
      ),
      'GetCommandRequest': grpc.unary_unary_rpc_method_handler(
          servicer.GetCommandRequest,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestResponse.SerializeToString,
      ),
      'GetCommandRequestStream': grpc.unary_stream_rpc_method_handler(
          servicer.GetCommandRequestStream,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestStreamRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.GetCommandRequestStreamResponse.SerializeToString,
      ),
      'SendCommandResponse': grpc.unary_unary_rpc_method_handler(
          servicer.SendCommandResponse,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.SendCommandResponseRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.SendCommandResponseResponse.SerializeToString,
      ),
      'PostTransformFrame': grpc.unary_unary_rpc_method_handler(
          servicer.PostTransformFrame,
          request_deserializer=protos_dot_model_dot_v1_dot_math__pb2.TransformFrame.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.PostTransformFrameResponse.SerializeToString,
      ),
      'SetBaseFrameID': grpc.unary_unary_rpc_method_handler(
          servicer.SetBaseFrameID,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.SetBaseFrameIDRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.SetBaseFrameIDResponse.SerializeToString,
      ),
      'ClearTransformTree': grpc.unary_unary_rpc_method_handler(
          servicer.ClearTransformTree,
          request_deserializer=protos_dot_agent_dot_v1_dot_agent__pb2.ClearTransformTreeRequest.FromString,
          response_serializer=protos_dot_agent_dot_v1_dot_agent__pb2.ClearTransformTreeResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'v1.agent.Agent', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import logging
import sys
import threading
import time
from typing import Any, Callable, Dict, List, Optional, Union

import grpc
from formant.protos.agent.v1 import agent_pb2, agent_pb2_grpc
from formant.protos.model.v1 import (
    commands_pb2,
    datapoint_pb2,
    event_pb2,
    math_pb2,
    media_pb2,
    navigation_pb2,
    text_pb2,
)
from typing_extensions import Literal

from .exceptions import handle_agent_exceptions


def current_timestamp():
    return int(time.time() * 1000)


class Client:
    """
    A client for interacting with the Formant Agent. There are methods for:
    - Ingesting telemetry datapoints
    - Ingesting transform frames
    - Creating events
    - Handling commands
    - Reading application configuration

    Automatically handles connection and reconnection to the agent.
    """

    def __init__(
        self,
        agent_url="unix:///var/lib/formant/agent.sock",  # type: str
        enable_logging=True,  # type: bool
        ignore_throttled=False,  # type: bool
        ignore_unavailable=False,  # type: bool
    ):

        if enable_logging:
            self.logger = logging.getLogger("formant")  # type: Optional[logging.Logger]
            self.logger.setLevel(logging.INFO)
            handler = logging.StreamHandler(sys.stdout)
            formatter = logging.Formatter("%(levelname)s: %(message)s")
            handler.setFormatter(formatter)
            self.logger.addHandler(handler)
        else:
            self.logger = None

        self.ignore_throttled = ignore_throttled
        self.ignore_unavailable = ignore_unavailable

        self._agent_url = agent_url  # type: str
        self._app_config = {}  # type: Dict[str, str]
        self._config_update_callbacks = []  # type: List[Callable[[], None]]
        self._command_request_callback_streams = (
            {}
        )  # type: Dict[Callable[..., None], grpc._channel._MultiThreadedRendezvous]

        self._setup_agent_communication()

    # Ingesting telemetry datapoints

    @handle_agent_exceptions
    def post_text(
        self,
        stream,  # type: str
        value,  # type: str
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post a text datapoint to a telemetry stream."""
        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                text=text_pb2.Text(value=value),
                tags=tags if tags else {},
                timestamp=timestamp if timestamp else current_timestamp(),
            )
        )

    @handle_agent_exceptions
    def post_json(
        self,
        stream,  # type: str
        value,  # type: str
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post a JSON datapoint to a telemetry stream."""
        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                json=text_pb2.Json(value=value),
                timestamp=timestamp if timestamp else int(time.time() * 1000),
                tags=tags if tags else {},
            )
        )

    @handle_agent_exceptions
    def post_numeric(
        self,
        stream,  # type: str
        value,  # type: Union[int, float]
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post a numeric datapoint to a telemetry stream."""
        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                numeric=math_pb2.Numeric(value=value),
                tags=tags if tags else {},
                timestamp=timestamp if timestamp else current_timestamp(),
            )
        )

    @handle_agent_exceptions
    def post_image(
        self,
        stream,  # type: str
        # raw bytes:
        value,  # type: str
        content_type="image/jpg",  # type: Literal["image/jpg", "image/png"]
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post an image datapoint to a telemetry stream."""
        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                image=media_pb2.Image(raw=value, content_type=content_type),
                tags=tags if tags else {},
                timestamp=timestamp if timestamp else current_timestamp(),
            )
        )

    @handle_agent_exceptions
    def post_bitset(
        self,
        stream,  # type: str
        bitset_dict,  # type: Dict[str, bool]
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post a bitset datapoint to a telemetry stream."""

        bitset = math_pb2.Bitset()
        for k, v in bitset_dict.items():
            bitset.bits.extend([math_pb2.Bit(key=k, value=v)])

        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                bitset=bitset,
                tags=tags if tags else {},
                timestamp=timestamp if timestamp else current_timestamp(),
            )
        )

    @handle_agent_exceptions
    def post_geolocation(
        self,
        stream,  # type: str
        latitude,  # type: float
        longitude,  # type: float
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Post a geolocation datapoint to a telemetry stream."""
        self.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=stream,
                location=navigation_pb2.Location(
                    latitude=latitude, longitude=longitude
                ),
                tags=tags if tags else {},
                timestamp=timestamp if timestamp else current_timestamp(),
            )
        )

    # Ingesting transform frames

    @handle_agent_exceptions
    def post_transform_frame(
        self,
        parent_frame,  # type: str
        child_frame,  # type: str
        tx,  # type: float
        ty,  # type: float
        tz,  # type: float
        rx,  # type: float
        ry,  # type: float
        rz,  # type: float
        rw,  # type: float
    ):
        # type: (...) -> None
        """Adds a transform frame, used to position datapoints in 3D space."""
        frame = math_pb2.TransformFrame()
        frame.parent_frame = parent_frame
        frame.child_frame = child_frame
        frame.transform.translation.x = tx
        frame.transform.translation.y = ty
        frame.transform.translation.z = tz
        frame.transform.rotation.x = rx
        frame.transform.rotation.y = ry
        frame.transform.rotation.z = rz
        frame.transform.rotation.w = rw
        self.agent_stub.PostTransformFrame(frame)

    # Creating events

    @handle_agent_exceptions
    def create_event(
        self,
        message,  # type: str
        notify=False,  # type: bool
        tags=None,  # type: Optional[Dict[str, str]]
        timestamp=None,  # type: Optional[int]
    ):
        """Creates and ingests an event."""
        self.agent_stub.CreateEvent(
            agent_pb2.CreateEventRequest(
                event=event_pb2.Event(
                    message=message,
                    notification_enabled=notify,
                    tags=tags if tags else {},
                    timestamp=timestamp if timestamp else current_timestamp(),
                )
            )
        )

    # Handling commands

    def get_command_request(
        self, command_filter=None  # type: Optional[List[str]]
    ):
        # type: (...) -> Optional[commands_pb2.CommandRequest]
        """
        If there is a command request in the agent's queue whose "command" value matches
        an element of the given command filter, takes and returns the command request.

        Otherwise, returns None if there are no matching command requests
        in the agent's queue.
        """
        command_request_request = agent_pb2.GetCommandRequestRequest(
            command_filter=command_filter
        )
        try:
            command_request = self.agent_stub.GetCommandRequest(
                command_request_request
            ).request
            return command_request if command_request.command else None
        except grpc.RpcError:
            return None

    def send_command_response(
        self,
        request_id,  # type: str
        success,  # type: bool
        datapoint=None,  # type: Optional[datapoint_pb2.Datapoint]
    ):
        # type: (...) -> None
        """
        Sends a command response for an identified command request to Formant.
        Returns an error if there was a problem sending the command response.
        """
        response = commands_pb2.CommandResponse(
            request_id=request_id, success=success, datapoint=datapoint,
        )
        request = agent_pb2.SendCommandResponseRequest(response=response)
        self.agent_stub.SendCommandResponse(request)

    def register_command_request_callback(
        self,
        f,  # type: Callable[[commands_pb2.CommandRequest], None]
        command_filter=None,
    ):
        # type : (...) -> None
        """
        Command requests issued to the agent whose "command" value matches an element
        of the given command filter will be streamed into the provided callback.
        """

        def target():
            # stream iterator will receive command requests until cancelled
            # via "stream.cancel()" in unregister_command_request_callback
            while True:
                try:
                    stream = self._get_command_request_stream(command_filter)
                    self._command_request_callback_streams[f] = stream
                    for _ in stream:
                        f(_.request)
                except grpc.RpcError as e:
                    if e.code() == grpc.StatusCode.CANCELLED:
                        return
                    # wait a short duration before re-registering when
                    # agent unavailable or other rpc error
                    time.sleep(0.1)

        threading.Thread(target=target, daemon=True).start()

    def unregister_command_request_callback(
        self, f,  # type: Callable[[commands_pb2.CommandRequest], None]
    ):
        # type : (...) -> None
        """
        Unregisters previously registered command request callbacks.
        """
        command_request_stream = self._command_request_callback_streams.pop(f, None)
        if command_request_stream is not None:
            # this call will raise grpc.StatusCode.CANCELLED
            command_request_stream.cancel()

    # Reading application configuration

    def get_app_config(self, key, *args):
        # type: (str, Any) -> Optional[str]
        """
        Returns the value for the given key that was set in
        Formant application configuration for this device,
        or returns the given default value.
        """
        if len(args) > 1:
            raise TypeError("function takes at most two args: (key: str, default: Any)")
        default = args[0] if len(args) == 1 else None

        if len(self._app_config.keys()) == 0:
            self._update_application_configuration()

        return self._app_config.get(key, default)

    def register_config_update_callback(self, f):
        # type: (Callable) -> None
        """
        Adds a function to the list of callbacks that are executed by the client
        when this device receives updated configuration from Formant.
        """
        self._config_update_callbacks.append(f)

    def unregister_config_update_callback(self, f):
        # type: (Callable) -> None
        """
        Removes a function from the list of callbacks that are executed by the client
        when this device receives updated configuration from Formant.
        """
        self._config_update_callbacks.remove(f)

    # Internal methods

    def _setup_agent_communication(self):
        self.channel = grpc.insecure_channel(self._agent_url)
        self.channel.subscribe(self._handle_connectivity_change, try_to_connect=True)
        self.agent_stub = agent_pb2_grpc.AgentStub(self.channel)

    def _handle_connectivity_change(self, connectivity):
        """Handle changes to gRPC channel connectivity."""
        if connectivity == grpc.ChannelConnectivity.READY:
            if self.logger:
                self.logger.info("Agent communication established.")
            self._update_application_configuration()
            self._run_update_config_callbacks()
        elif connectivity == grpc.ChannelConnectivity.SHUTDOWN:
            # In the case of shutdown, re-establish the connection from scratch
            if self.logger:
                self.logger.info("Agent communication lost. Re-establishing...")
            self._setup_agent_communication()

    @handle_agent_exceptions
    def _update_application_configuration(self):
        request = agent_pb2.GetApplicationConfigurationRequest()
        response = self.agent_stub.GetApplicationConfiguration(request)
        self._app_config = response.configuration.configuration_map

    def _run_update_config_callbacks(self):
        [f() for f in self._config_update_callbacks]

    def _get_command_request_stream(self, command_filter=None):
        return self.agent_stub.GetCommandRequestStream(
            agent_pb2.GetCommandRequestStreamRequest(command_filter=command_filter)
        )
